//  Controls.h - a class for handling control events.
//
//	Vamos Automotive Simulator
//  Copyright (C) 2003 Sam Varner
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef _CONTROLS_H_
#define _CONTROLS_H_

#include <sigc++/sigc++.h>

#include <vector>
#include <map>

class jsJoystick;

namespace Vamos_World
{
  class Gl_World;

  struct Bad_Axis
  {
	Bad_Axis (int ax, int num) : axis (ax), num_axes (num) {};
	int axis;
	int num_axes;
  };

  // Derive from SigC::Object so member functions can be GL callbacks.
  class Controls : public SigC::Object
  {
  public:
	typedef bool (Gl_World::* Callback_Func) (double, double);

  private:
	struct Callback
	{
	  Callback (Gl_World* obj = 0, 
				Callback_Func func = 0, 
				double d1 = 0.0,
				double d2 = 0.0,
				double d3 = 0.0)
		: object (obj), function (func), data1 (d1), data2 (d2), data3 (d3) {};

	  Gl_World* object;
	  Callback_Func function;
	  double data1;
	  double data2;
	  double data3;
	};

	std::map <unsigned char, Callback> m_key_down_map;
	std::map <unsigned char, Callback> m_key_up_map;
	std::map <int, Callback> m_special_key_down_map;
	std::map <int, Callback> m_special_key_up_map;

	std::vector <std::pair <int, Callback> > m_button_down_map;
	std::vector <std::pair <int, Callback> > m_button_up_map;
	std::vector <std::pair <int, Callback> > m_axis_map;

	jsJoystick* mp_js_joystick;
	bool m_joystick_is_present;
	int m_num_axes;
	float* mp_js_axes;
	bool m_auto_clutch;
	int m_last_buttons;

  public:
	Controls ();
	~Controls ();

	void bind_key_down (unsigned char key, 
						Gl_World* world, 
						Callback_Func func,
						double time = 0.0);
	void bind_key_up (unsigned char key, 
					  Gl_World* world, 
					  Callback_Func func,
					  double time = 0.0);
	void bind_special_key_down (int key, 
								Gl_World* world, 
								Callback_Func func,
								double time = 0.0);
	void bind_special_key_up (int key, 
							  Gl_World* world, 
							  Callback_Func func,
							  double time = 0.0);

	void bind_button_down (int button, 
						   Gl_World* world, 
						   Callback_Func func,
						   double time = 0.0);
	void bind_button_up (int button, 
						 Gl_World* world, 
						 Callback_Func func,
						 double time = 0.0);
	void bind_axis (int axis, 
					Gl_World* world, 
					Callback_Func func, 
					double factor = 1.0,
					double offset = 0.0,
					double minimum = 0.0,
					double deadband = 0.0);
				
	void key_press (unsigned char key, int, int);
	void key_release (unsigned char key, int, int);
	void special_key_press (int key, int, int);
	void special_key_release (int key, int, int);

	void update ();
  };
}

#endif // not _CONTROLS_H_
